@@ -37,6 +37,7 @@
 #include "mapcommon.h"
 #include "mapfetcher.h"
 #include "mapscene.h"
+#include "mapscroller.h"
 #include "maptile.h"
 #include "network/networkaccessmanager.h"
 #include "ownlocationitem.h"
@@ -44,13 +45,17 @@
 
 #include "mapengine.h"
 
+const int SMOOTH_CENTERING_TIME_MS = 1000;
+
 MapEngine::MapEngine(QObject *parent)
     : QObject(parent),
       m_autoCenteringEnabled(false),
+      m_scrollStartedByGps(false),
+      m_smoothScrollRunning(false),
       m_zoomedIn(false),
       m_zoomLevel(DEFAULT_ZOOM_LEVEL),
       m_centerTile(QPoint(UNDEFINED, UNDEFINED)),
-      m_lastManualPosition(QPoint(0, 0)),
+      m_lastAutomaticPosition(QPoint(0, 0)),
       m_tilesGridSize(QSize(0, 0)),
       m_viewSize(QSize(DEFAULT_SCREEN_WIDTH, DEFAULT_SCREEN_HEIGHT))
 {
@@ -63,8 +68,8 @@ MapEngine::MapEngine(QObject *parent)
             m_mapFetcher, SLOT(enqueueFetchMapImage(int, int, int)));
     connect(m_mapFetcher, SIGNAL(mapImageReceived(int, int, int, QPixmap)),
             this, SLOT(mapImageReceived(int, int, int, QPixmap)));
-    connect(m_mapFetcher, SIGNAL(error(int)),
-            this, SIGNAL(error(int)));
+    connect(m_mapFetcher, SIGNAL(error(int, int)),
+            this, SIGNAL(error(int, int)));
 
     m_ownLocation = new OwnLocationItem();
     m_ownLocation->hide(); // hide until first location info is received
@@ -85,6 +90,14 @@ MapEngine::MapEngine(QObject *parent)
 
     connect(m_friendItemsHandler, SIGNAL(locationItemClicked(QList<QString>)),
             this, SIGNAL(locationItemClicked(QList<QString>)));
+
+    m_scroller = &MapScroller::getInstance();
+
+    connect(m_scroller, SIGNAL(coordinateUpdated(QPoint)),
+            this, SLOT(setCenterPosition(QPoint)));
+
+    connect(m_scroller, SIGNAL(stateChanged(QAbstractAnimation::State, QAbstractAnimation::State)),
+            this, SLOT(scrollerStateChanged(QAbstractAnimation::State)));
 }
 
 MapEngine::~MapEngine()
@@ -117,6 +130,13 @@ QPointF MapEngine::centerGeoCoordinate()
     return convertSceneCoordinateToLatLon(m_zoomLevel, m_sceneCoordinate);
 }
 
+void MapEngine::centerToCoordinates(QPointF latLonCoordinate)
+{
+    qDebug() << __PRETTY_FUNCTION__;
+
+    scrollToPosition(convertLatLonToSceneCoordinate(latLonCoordinate));
+}
+
 QPoint MapEngine::convertLatLonToSceneCoordinate(QPointF latLonCoordinate)
 {
     qDebug() << __PRETTY_FUNCTION__;
@@ -177,23 +197,23 @@ QPoint MapEngine::convertTileNumberToSceneCoordinate(int zoomLevel, QPoint tileN
     return QPoint(x, y);
 }
 
-bool MapEngine::disableAutoCentering(QPoint sceneCoordinate)
+void MapEngine::disableAutoCenteringIfRequired(QPoint sceneCoordinate)
 {
     if (isAutoCenteringEnabled()) {
         int zoomFactor = (1 << (MAX_MAP_ZOOM_LEVEL - m_zoomLevel));
 
-        QPoint oldPixelValue = QPoint(m_lastManualPosition.x() / zoomFactor,
-                                      m_lastManualPosition.y() / zoomFactor);
+        QPoint oldPixelValue = QPoint(m_lastAutomaticPosition.x() / zoomFactor,
+                                      m_lastAutomaticPosition.y() / zoomFactor);
 
         QPoint newPixelValue = QPoint(sceneCoordinate.x() / zoomFactor,
                                       sceneCoordinate.y() / zoomFactor);
 
-        if ((abs(oldPixelValue.x() - newPixelValue.x()) > AUTO_CENTERING_DISABLE_DISTANCE) ||
-            (abs(oldPixelValue.y() - newPixelValue.y()) > AUTO_CENTERING_DISABLE_DISTANCE))
-            return true;
-    }
+        if ((abs(oldPixelValue.x() - newPixelValue.x()) > AUTO_CENTERING_DISABLE_DISTANCE)
+            || (abs(oldPixelValue.y() - newPixelValue.y()) > AUTO_CENTERING_DISABLE_DISTANCE)) {
 
-    return false;
+            emit mapScrolledManually();
+        }
+    }
 }
 
 void MapEngine::friendsPositionsUpdated()
@@ -237,8 +257,11 @@ void MapEngine::gpsPositionUpdate(QPointF position, qreal accuracy)
     m_gpsLocationItem->updatePosition(convertLatLonToSceneCoordinate(position), accuracy);
     m_mapScene->spanItems(m_zoomLevel, m_sceneCoordinate, m_viewSize);
 
-    if (m_autoCenteringEnabled)
-        setViewLocation(position);
+    if (m_autoCenteringEnabled) {
+        m_lastAutomaticPosition = convertLatLonToSceneCoordinate(position);
+        m_scrollStartedByGps = true;
+        scrollToPosition(m_lastAutomaticPosition);
+    }
 }
 
 qreal MapEngine::greatCircleDistance(QPointF firstLocation, QPointF secondLocation)
@@ -276,7 +299,7 @@ void MapEngine::init()
     }
 
     emit zoomLevelChanged(m_zoomLevel);
-    setViewLocation(QPointF(startLocation.x(), startLocation.y()));
+    centerToCoordinates(QPointF(startLocation.x(), startLocation.y()));
 }
 
 bool MapEngine::isAutoCenteringEnabled()
@@ -374,37 +397,60 @@ QGraphicsScene* MapEngine::scene()
     return m_mapScene;
 }
 
+void MapEngine::scrollerStateChanged(QAbstractAnimation::State newState)
+{
+    qDebug() << __PRETTY_FUNCTION__;
+
+    if (m_smoothScrollRunning
+        && newState != QAbstractAnimation::Running) {
+            m_smoothScrollRunning = false;
+
+            // don't disable auto centering if current animation was stopped by new update from GPS
+            if (!m_scrollStartedByGps)
+                disableAutoCenteringIfRequired(m_sceneCoordinate);
+    }
+
+    m_scrollStartedByGps = false;
+}
+
+void MapEngine::scrollToPosition(QPoint scenePosition)
+{
+    qDebug() << __PRETTY_FUNCTION__;
+
+    m_scroller->stop();
+    m_scroller->setEasingCurve(QEasingCurve::InOutQuart);
+    m_scroller->setDuration(SMOOTH_CENTERING_TIME_MS);
+    m_scroller->setStartValue(m_sceneCoordinate);
+    m_scroller->setEndValue(scenePosition);
+    m_smoothScrollRunning = true;
+    m_scroller->start();
+}
+
 void MapEngine::setAutoCentering(bool enabled)
 {
+    qDebug() << __PRETTY_FUNCTION__;
+
     m_autoCenteringEnabled = enabled;
 }
 
-void MapEngine::setGPSEnabled(bool enabled)
-{
-    m_gpsLocationItem->setEnabled(enabled);
-}
-
-void MapEngine::setLocation(QPoint sceneCoordinate)
+void MapEngine::setCenterPosition(QPoint scenePosition)
 {
     qDebug() << __PRETTY_FUNCTION__;
 
     // jump to opposite side of the world if world horizontal limit is exceeded
-    if (sceneCoordinate.x() < MAP_MIN_PIXEL_X)
-        sceneCoordinate.setX(sceneCoordinate.x() + MAP_PIXELS_X);
-    else if (sceneCoordinate.x() > MAP_MAX_PIXEL_X)
-        sceneCoordinate.setX(sceneCoordinate.x() - MAP_PIXELS_X);
+    scenePosition.setX(normalize(scenePosition.x(), MAP_MIN_PIXEL_X, MAP_MAX_PIXEL_X));
 
     // don't allow vertical scene coordinates go out of the map
-    sceneCoordinate.setY(qBound(MAP_MIN_PIXEL_Y, sceneCoordinate.y(), MAP_MAX_PIXEL_Y));
+    scenePosition.setY(qBound(MAP_MIN_PIXEL_Y, scenePosition.y(), MAP_MAX_PIXEL_Y));
 
-    if (disableAutoCentering(sceneCoordinate))
-        emit mapScrolledManually();
+    if (!m_smoothScrollRunning)
+        disableAutoCenteringIfRequired(scenePosition);
 
-    m_sceneCoordinate = sceneCoordinate;
+    m_sceneCoordinate = scenePosition;
     emit locationChanged(m_sceneCoordinate);
 
-    if (isCenterTileChanged(sceneCoordinate)) {
-        getTiles(sceneCoordinate);
+    if (isCenterTileChanged(scenePosition)) {
+        getTiles(scenePosition);
         m_mapScene->removeOutOfViewTiles(m_viewTilesGrid, m_zoomLevel);
     }
 
@@ -412,6 +458,13 @@ void MapEngine::setLocation(QPoint sceneCoordinate)
     emit newMapResolution(sceneResolution());
 }
 
+void MapEngine::setGPSEnabled(bool enabled)
+{
+    qDebug() << __PRETTY_FUNCTION__;
+
+    m_gpsLocationItem->setEnabled(enabled);
+}
+
 void MapEngine::setZoomLevel(int newZoomLevel)
 {
     qDebug() << __PRETTY_FUNCTION__;
@@ -442,17 +495,6 @@ void MapEngine::setTilesGridSize(const QSize &viewSize)
     m_tilesGridSize.setWidth(gridWidth);
 }
 
-void MapEngine::setViewLocation(QPointF latLonCoordinate)
-{
-    qDebug() << __PRETTY_FUNCTION__;
-
-    QPoint sceneCoordinate = convertLatLonToSceneCoordinate(latLonCoordinate);
-
-    m_lastManualPosition = sceneCoordinate;
-
-    setLocation(sceneCoordinate);
-}
-
 int MapEngine::tileMaxIndex(int zoomLevel)
 {
     qDebug() << __PRETTY_FUNCTION__;
