@@ -20,6 +20,7 @@
 */
 
 #include <QDebug>
+#include <QTimer>
 
 #include "networkhandler.h"
 #include "networkhandlerprivatestub.h"
@@ -29,6 +30,9 @@ NetworkHandlerPrivate::NetworkHandlerPrivate(QObject *parent)
     : QObject(parent)
 {
     qDebug() << __PRETTY_FUNCTION__;
+
+    m_parent = static_cast<NetworkHandler*>(parent);
+    QTimer::singleShot(0, this, SLOT(connected()));
 }
 
 void NetworkHandlerPrivate::connect()
@@ -36,6 +40,13 @@ void NetworkHandlerPrivate::connect()
     qDebug() << __PRETTY_FUNCTION__;
 }
 
+void NetworkHandlerPrivate::connected()
+{
+    qDebug() << __PRETTY_FUNCTION__;
+
+    emit m_parent->connected();
+}
+
 void NetworkHandlerPrivate::disconnect()
 {
     qDebug() << __PRETTY_FUNCTION__;
