@@ -32,6 +32,7 @@
 #include "map/mapview.h"
 #include "common.h"
 #include "friendlistpanel.h"
+#include "fullscreenbutton.h"
 #include "logindialog.h"
 #include "mapscale.h"
 #include "settingsdialog.h"
@@ -54,6 +55,7 @@ MainWindow::MainWindow(QWidget *parent)
     m_errorShown(false),
     m_loggedIn(false),
     m_refresh(false),
+    m_progressIndicatorCount(0),
     m_ownLocationCrosshair(0),
     m_email(),    
     m_password(),
@@ -93,8 +95,6 @@ MainWindow::MainWindow(QWidget *parent)
     m_mapScale->stackUnder(m_ownLocationCrosshair);
     m_mapView->stackUnder(m_mapScale);
 
-    this->toggleProgressIndicator(true);
-
     grabZoomKeys(true);
 
     // Set default screen size
@@ -139,9 +139,7 @@ void MainWindow::buildFullScreenButton()
 {
     qDebug() << __PRETTY_FUNCTION__;
 #ifdef Q_WS_MAEMO_5
-    m_fullScreenButton = new QToolButton(this);
-    m_fullScreenButton->setIcon(QIcon::fromTheme(QLatin1String("general_fullsize")));
-    m_fullScreenButton->setFixedSize(m_fullScreenButton->sizeHint());
+    m_fullScreenButton = new FullScreenButton(this);
     connect(m_fullScreenButton, SIGNAL(clicked()),
             this, SLOT(toggleFullScreen()));
 #endif // Q_WS_MAEMO_5
@@ -255,6 +253,9 @@ void MainWindow::buildMap()
 
     connect(m_mapView, SIGNAL(viewZoomFinished()),
             this, SIGNAL(viewZoomFinished()));
+
+    connect(m_mapView, SIGNAL(zoomIn()),
+            this, SIGNAL(zoomIn()));
 }
 
 void MainWindow::buildMapScale()
@@ -472,9 +473,14 @@ void MainWindow::drawFullScreenButton(const QSize &size)
     qDebug() << __PRETTY_FUNCTION__ << size.width() << "x" << size.height();
 
     if(m_fullScreenButton) {
-        m_fullScreenButton->move(size.width() - m_fullScreenButton->size().width()
-                                 - PANEL_PEEK_AMOUNT,
-                                 size.height() - m_fullScreenButton->size().height());
+        if(m_loggedIn) {
+            m_fullScreenButton->move(size.width() - m_fullScreenButton->size().width()
+                                     - PANEL_PEEK_AMOUNT,
+                                     size.height() - m_fullScreenButton->size().height());
+        } else {
+            m_fullScreenButton->move(size.width() - m_fullScreenButton->size().width(),
+                                     size.height() - m_fullScreenButton->size().height());
+        }
     }
 }
 
@@ -665,9 +671,6 @@ void MainWindow::loginFailed()
     qDebug() << __PRETTY_FUNCTION__;
 
     clearCookieJar();
-
-    toggleProgressIndicator(false);
-
     startLoginProcess();
 }
 
@@ -682,6 +685,8 @@ void MainWindow::loginUsingCookies()
 {
     qDebug() << __PRETTY_FUNCTION__;
 
+    toggleProgressIndicator(true);
+
     buildWebView();
     loadCookies();
     
@@ -797,7 +802,7 @@ void MainWindow::setOwnLocationCrosshairVisibility(bool visibility)
     if (visibility && m_loggedIn) {
         m_ownLocationCrosshair->show();
         m_drawOwnLocationCrosshair = true;
-        drawOwnLocationCrosshair(QSize(m_viewPortWidth, m_viewPortHeight));
+        drawOwnLocationCrosshair(m_viewPortSize);
     } else {
         m_ownLocationCrosshair->hide();
         m_drawOwnLocationCrosshair = false;
@@ -822,8 +827,7 @@ void MainWindow::setViewPortSize(const QSize &size)
 {
     qDebug() << __PRETTY_FUNCTION__;
 
-    m_viewPortWidth = size.width();
-    m_viewPortHeight = size.height();
+    m_viewPortSize = size;
 }
 
 void MainWindow::showEnableAutomaticUpdateLocationDialog(const QString &text)
@@ -875,6 +879,25 @@ void MainWindow::showInformationBox()
     }
 }
 
+void MainWindow::showPanels()
+{
+    qDebug() << __PRETTY_FUNCTION__;
+
+    drawFullScreenButton(m_viewPortSize);
+
+    if(m_loggedIn) {
+        if(!m_friendsListPanel->isVisible()) {
+            m_friendsListPanel->show();
+            m_friendsListPanelSidebar->show();
+        }
+
+        if(!m_userPanel->isVisible()) {
+            m_userPanel->show();
+            m_userPanelSidebar->show();
+        }
+    }
+}
+
 void MainWindow::startLoginProcess()
 {
     qDebug() << __PRETTY_FUNCTION__;
@@ -896,7 +919,16 @@ void MainWindow::toggleProgressIndicator(bool value)
     qDebug() << __PRETTY_FUNCTION__;
 
 #ifdef Q_WS_MAEMO_5
-    setAttribute(Qt::WA_Maemo5ShowProgressIndicator, value);
+    if(value) {
+        m_progressIndicatorCount++;
+        setAttribute(Qt::WA_Maemo5ShowProgressIndicator, true);
+    } else {
+        if(m_progressIndicatorCount > 0)
+            m_progressIndicatorCount--;
+
+        if(m_progressIndicatorCount == 0)
+            setAttribute(Qt::WA_Maemo5ShowProgressIndicator, false);
+    }
 #else
     Q_UNUSED(value);
 #endif // Q_WS_MAEMO_5
@@ -906,11 +938,7 @@ void MainWindow::updateItemVisibility()
 {
     qDebug() << __PRETTY_FUNCTION__;
     
-    if(m_loggedIn) {
-        m_friendsListPanel->show();
-        m_friendsListPanelSidebar->show();
-        m_userPanel->show();
-        m_userPanelSidebar->show();
+    if(m_loggedIn) {       
 
         if(!m_gpsToggleAct->isChecked())
             setOwnLocationCrosshairVisibility(true);
@@ -940,8 +968,7 @@ void MainWindow::webViewRequestFinished(QNetworkReply *reply)
     // qwebview starts to load a new page while the current page loading is not finished
     if(reply->error() != QNetworkReply::OperationCanceledError &&
        reply->error() != QNetworkReply::NoError) {
-        emit error(reply->error());
-        toggleProgressIndicator(false);
+        emit error(ErrorContext::NETWORK, reply->error());
     }
 }
 
